"""Models package for linear regression implementations."""

from .linear_regression import LinearRegression
from .polynomial_regression import PolynomialRegression

__all__ = [
    "LinearRegression",
    "PolynomialRegression",
]
